/*______________________________________________________________________________
 * 
 * org.eidola.kernel.Compiler
 * 
 * Part of the Eidola Kernel Reference Implementation
 * See http://eidola.org for oodles of relevant fun!
 * 
 *______________________________________________________________________________
 * 
 * Copyright 2000-2001 Paul Cantrell
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License version 2, as published by the
 * Free Software Foundation. See the file LICENSE.html for more information.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY, including the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc. / 59 Temple
 * Place, Suite 330 / Boston, MA 02111-1307 / USA.
 *_______________________________________________________________________________
 */

package org.eidola.kernel;

import org.eidola.kernel.event.*;

/**
	An EventListener which compiles containers as they broadcast {@link CompileRequired}
	events. {@link Engine} creates one of these and attaches a queue and a group of threads.
	<p>
	The compilation process runs as follows:
	<ul>
		<li> An container broadcasts a {@link CompileRequired} event.
		<li> {@link Compiler} picks up the event, and calls {@link Container#compile()}.
		<li> {@link Container#compile()} creates a new {@link Container.Compilation Compilation},
			and calls {@link Container.Compilation#run()}.
		<li> If the container is modified while {@link Container#compile()} is running, it
			throws {@link CompileAbortedException}.
		<li> If the compilation isn't aborted, a {@link CompileCompleted} event goes out,
			and subsequent calls to {@link Container#getCompilation()}
			return the newly compeleted {@link Container.Compilation Compilation}.
	</ul>
	<p>
	@see Container#compile()
	@see Container.Compilation
	@see CompileRequired
	@see Engine
	
	@author Paul Cantrell
	@version [Development version]
*/

public class Compiler
	implements EventListener
	{
	/** If the event is a {@link CompileRequired}, this handler
	 *  calls {@link Container#compile()} for the event's source. */
	
	public void handleEvent(Event event)
		{
		if(event instanceof CompileRequired)
			try
				{ ((CompileRequired) event).getSource().compile(); }
			catch(CompileAbortedException cae)
				{ /* We don't care -- we'll see the event which aborted it in a moment! */ }
		}
	
	// Yes, it really is this simple.
	}





