/*______________________________________________________________________________
 * 
 * org.eidola.kernel.Package
 * 
 * Part of the Eidola Kernel Reference Implementation
 * See http://eidola.org for oodles of relevant fun!
 * 
 *______________________________________________________________________________
 * 
 * Copyright 1998, 2000-2001 Paul Cantrell
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License version 2, as published by the
 * Free Software Foundation. See the file LICENSE.html for more information.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY, including the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc. / 59 Temple
 * Place, Suite 330 / Boston, MA 02111-1307 / USA.
 *_______________________________________________________________________________
 */

package org.eidola.kernel;

import org.eidola.util.Util;
import java.util.*;

/**
	The largest unit in Eidola's hierarchical namespace.
	Packages serve several purposes:
	<ul>
		<li> to group conceptually related elements into a module,
		<li> to prevent namespace collisions between modules, and
		<li> to allow modules to encapsulate on a scale larger than classes.
	</ul>
	<p>
	<b>Paul pontificates:</b>
	Packages can currently be members of classes.  This may be fine in a basic
	structural sense, but I'm not sure what it means at runtime, or whether
	it means anything.  A package within a class is probably just a way of
	doing internal encapsulation -- probably an excellent idea (though there may
	be better ways of doing it).  If this is what it means, the signature propagation
	rules need to change.  It might also be a way of doing static members,
	in which case the current signature rules are OK, but again there's probably
	a better way of doing this.
	<p>
	<font size=+1><b>Structure:</b></font>
	<ul>
		<li> name
		<li> owner
		<li> public members
		<li> private members
	</ul>
	<p>
	<font size=+1><b>Signature:</b></font>
	<ul>
		<li> packages only specialize themselves, so nothing affects the signature
	</ul>
	<p>
	See note on synchronization and concurrent read safety in {@link Container}.
	
	@author Paul Cantrell
	@version [Development version]
*/

public class Package
	extends Capsule
	{
	//------------------------------------------------------
	//  Constructors
	//------------------------------------------------------
	
	/** Creates a new empty package. */
	
	public Package()
		{ }
	
	//------------------------------------------------------
	//  Semantic conditions and errors
	//------------------------------------------------------
	
	protected Container.Compilation makeNewCompilation()
		{ return new Compilation(); }
		
	public class Compilation
		extends Capsule.Compilation
		{
		/** A package only specializes itself. */
		
		public boolean specializes(NamedElement other)
			{ return other == Package.this; }
		}

	//------------------------------------------------------
	//  Debugging
	//------------------------------------------------------
	
	/** Dumps the structure of the element to System.out.
	 *  @param i Number of spaces to Util.indent */
	
	public void dump(int i)
		{
		Util.indent(i);
		
		System.out.print("Package " + toString());
		if(!getCompilation().isCurrent())
			System.out.print("  ** compile needed **");
		System.out.print('\n');
		
		Util.indent(i+2);
		System.out.print("public:\n");
		for(Iterator elem = getPublic().iterator(); elem.hasNext(); )
			((Element) elem.next()).dump(i+4);

		Util.indent(i+2);
		System.out.print("private:\n");
		for(Iterator elem = getPrivate().iterator(); elem.hasNext(); )
			((Element) elem.next()).dump(i+4);
		}
	}
