/*______________________________________________________________________________
 * 
 * org.eidola.kernel.event.CompileRequired
 * 
 * Part of the Eidola Kernel Reference Implementation
 * See http://eidola.org for oodles of relevant fun!
 * 
 *______________________________________________________________________________
 * 
 * Copyright 2000-2001 Paul Cantrell
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License version 2, as published by the
 * Free Software Foundation. See the file LICENSE.html for more information.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY, including the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc. / 59 Temple
 * Place, Suite 330 / Boston, MA 02111-1307 / USA.
 *_______________________________________________________________________________
 */

package org.eidola.kernel.event;

import org.eidola.kernel.Container;
import java.util.*;

/**
	Signals that a container needs to be recompiled.
	<p>
	This is a generalization for other events, such as {@link StructureChanged} and
	{@link SignatureChanged}, which require recompilation but convey more specific
	information.  This class also works alone for forcing recompilation of a
	conatiner without signaling any changes to it.
	<p>
	CompileEvents can propagate other compile events.  For example, if the name of a 
	parameter of a function changes, the structure change in the parameter propagates
	a signature change in the parameter, which in turn propagates a signature change
	in the function, which might require recompilation of a class of which the
	function is a member.
	
	@author Paul Cantrell
	@version [Development version]
*/

public class CompileRequired
	extends ContainerEvent
	{
	/**
	 * Creates a new compile required event which is the start of a propagation chain.
	 * @param source The element which needs to be recompiled.
	 */
	
	public CompileRequired(Container source)
		{ super(source); }
	
	/**
	 * Creates a compile required event which was propagated from another such event.
	 *
	 * @param source The element which needs to be recompiled.
	 * @param propagatedFrom The event which propagated this one.
	 */
	
	public CompileRequired(Container source, ContainerEvent propagatedFrom)
		{ super(source, propagatedFrom); }
	}


