/*______________________________________________________________________________
 * 
 * org.eidola.kernel.event.ContainerEvent
 * 
 * Part of the Eidola Kernel Reference Implementation
 * See http://eidola.org for oodles of relevant fun!
 * 
 *______________________________________________________________________________
 * 
 * Copyright 2000-2001 Paul Cantrell
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License version 2, as published by the
 * Free Software Foundation. See the file LICENSE.html for more information.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY, including the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc. / 59 Temple
 * Place, Suite 330 / Boston, MA 02111-1307 / USA.
 *_______________________________________________________________________________
 */

package org.eidola.kernel.event;

import org.eidola.kernel.Container;
import java.util.*;

/**
	An event originating from a container.
	
	@see Event
	@see Container

	@author Paul Cantrell
	@version [Development version]
*/

public abstract class ContainerEvent
	implements Event
	{
	/** Creates a new blank element event.
	 *  @param source The element which sent the event. */
	
	public ContainerEvent(Container source)
		{ this(source, null); }
	
	/** Creates a new blank element event which was spawned by another event.
	 *  @param source The element which sent the event. */
	
	public ContainerEvent(Container source, ContainerEvent propagatedFrom)
		{
		this.source = source;
		this.propagatedFrom = propagatedFrom;
		}
	
	/** Returns the element which caused this event. */
	
	public Container getSource()
		{ return source; }
	
	/** Returns the event which propagated this one, or null if this event was
	 *  spawned by an action from outside the kernel. */
	
	public ContainerEvent getPropagatedFrom()
		{ return propagatedFrom; }
	
	/** Determines whether a given element is involved in the chain of events which
	 *  lead to this event.  Useful for detecting cycles in event triggering, such as
	 *  when two elements' signatures affect one another. */
	
	public boolean propagationChainContains(Event event)
		{
		for(ContainerEvent curEvt = this; curEvt != null; curEvt = curEvt.getPropagatedFrom())
			if(curEvt.equals(event))
				return true;
		return false;
		}
	
	public String toString()
		{
		String str = getClass().getName() + " source=("+source+")";
		for(ContainerEvent p = getPropagatedFrom(); p != null; p = p.getPropagatedFrom())
			str += "(" + p.getSource() + ")";
		return str;
		}
	
	public int hashCode()
		{ return source.hashCode() * 13 + getClass().hashCode(); }
	
	/** Two ContainerEvents are equal if they have the same dynamic type, and
	 *  came from the same source element. */
	
	public boolean equals(Object other)
		{
		if(other == null)
			return false;
		return getClass() == other.getClass()
			&& source == ((ContainerEvent) other).source;
		}
	
	private Container source;
	private ContainerEvent propagatedFrom;
	}



