/*______________________________________________________________________________
 * 
 * org.eidola.kernel.event.Event
 * 
 * Part of the Eidola Kernel Reference Implementation
 * See http://eidola.org for oodles of relevant fun!
 * 
 *______________________________________________________________________________
 * 
 * Copyright 2000-2001 Paul Cantrell
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License version 2, as published by the
 * Free Software Foundation. See the file LICENSE.html for more information.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY, including the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc. / 59 Temple
 * Place, Suite 330 / Boston, MA 02111-1307 / USA.
 *_______________________________________________________________________________
 */

package org.eidola.kernel.event;

import org.eidola.kernel.Element;

/**
	Carries information about changes, updates, etc.  Events originate
	from {@link EventBroadcaster}s, and objects implementing
	the {@link EventListener} interface may register themselves to receive
	events.
	<p>
	The lifespan of an event typically runs as follows:
	<ul>
		<li> {@link EventListener}s register themselves to receive events via
			{@link EventBroadcaster#addListener(EventListener,EventQueue)}.
		<li> Some fascinating occurrence causes an object to create a new {@link Event},
			and call {@link EventBroadcaster#broadcastEvent(Event)}.
		<li> The broadcaster calls {@link EventQueue#addEvent(Event event, EventListener target)}
			for each registered listener.
		<li> Threads are consuming events from the various queues, and dispatch the events
			to the appropriate listeners with calls to {@link EventListener#handleEvent(Event)}.
	</ul>
	<p>
	Subclasses of Event may override the {@link java.lang.Object#equals(Object) equals}
	and {@link java.lang.Object#hashCode() hashCode} methods to reduce redundant
	event broadcasting.  See {@link EventQueue} for more information.
	
	@see EventBroadcaster
	@see EventListener
	@see EventQueue
	@see ContainerEvent
	
	@author Paul Cantrell
	@version [Development version]
*/

public interface Event
	{
	}



