/*______________________________________________________________________________
 * 
 * org.eidola.kernel.event.GeneralizationsChanged
 * 
 * Part of the Eidola Kernel Reference Implementation
 * See http://eidola.org for oodles of relevant fun!
 * 
 *______________________________________________________________________________
 * 
 * Copyright 2000-2001 Paul Cantrell
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License version 2, as published by the
 * Free Software Foundation. See the file LICENSE.html for more information.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY, including the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc. / 59 Temple
 * Place, Suite 330 / Boston, MA 02111-1307 / USA.
 *_______________________________________________________________________________
 */

package org.eidola.kernel.event;

import org.eidola.kernel.Class;
import org.eidola.util.*;

/**
	Signals that a class's generalizations have changed.
	Note that this event is not a CompileRequired.  This is because it's sent
	only after compilation; although it may require recompilation of other elements
	which refernce the class that changed, the class itself is already up-to-date.
	
	@author Paul Cantrell
	@version [Development version]
*/

public class GeneralizationsChanged
	extends ContainerEvent
	{
	public GeneralizationsChanged(Class source, CollectionDiff changes)
		{
		super(source);
		this.changes = changes;
		}
	
	public CollectionDiff getChanges()
		{ return changes; }
	
	/** Returns true if other is also StructureChanged with the same source
	 *  and the same changes. */
	
	public boolean equals(Object other)
		{
		return super.equals(other) // Checks class equality
			&& changes.equals(((GeneralizationsChanged) other).changes);
		}
	
	public String toString()
		{
		return super.toString()
			+ " changes=(+["
			+ Util.collectionToString(changes.getAdded(), " ") + "] -["
			+ Util.collectionToString(changes.getRemoved(), " ") + "])";
		}
	
	private CollectionDiff changes;
	}