/*______________________________________________________________________________
 * 
 * org.eidola.kernel.event.StructureChanged
 * 
 * Part of the Eidola Kernel Reference Implementation
 * See http://eidola.org for oodles of relevant fun!
 * 
 *______________________________________________________________________________
 * 
 * Copyright 2000-2001 Paul Cantrell
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License version 2, as published by the
 * Free Software Foundation. See the file LICENSE.html for more information.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY, including the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc. / 59 Temple
 * Place, Suite 330 / Boston, MA 02111-1307 / USA.
 *_______________________________________________________________________________
 */

package org.eidola.kernel.event;

import org.eidola.kernel.Container;
import org.eidola.kernel.ContainerPart;

/**
	Signals that the source container's structure has changed.
	<p>
	Structure changes never propagate from other events; their propagator is
	always null.
	
	@see Container
	
	@author Paul Cantrell
	@version [Development version]
*/

public class StructureChanged
	extends CompileRequired
	{
	/** @param source The element whose structure has changed.
	 *  @param part Which part of the element's structure has changed (e.g.
	 *  	{@link org.eidola.kernel.Element#OWNER}). */
	
	public StructureChanged(Container source, ContainerPart part)
		{
		super(source);
		this.part = part;
		}
	
	/** Returns which part of the element's structure has changed
	 *  (e.g.&nbsp;{@link org.eidola.kernel.Element#OWNER}) */
	
	public ContainerPart getPart()
		{ return part; }
	
	public String toString()
		{ return super.toString() + " part=(" + part.getName() + ")"; }
	
	public int hashCode()
		{ return super.hashCode() * 6061 + part.hashCode(); }
	
	/** Returns true if other is also StructureChanged with the same source
	 *  and the same part. */
	
	public boolean equals(Object other)
		{
		return super.equals(other) // Checks class equality
			&& part == ((StructureChanged) other).part;
		}
	
	private ContainerPart part;
	}


