/*______________________________________________________________________________
 * 
 * org.eidola.test.Test
 * 
 * Part of the Eidola Kernel Reference Implementation
 * See http://eidola.org for oodles of relevant fun!
 * 
 *______________________________________________________________________________
 * 
 * Copyright 2001 Paul Cantrell
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License version 2, as published by the
 * Free Software Foundation. See the file LICENSE.html for more information.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY, including the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc. / 59 Temple
 * Place, Suite 330 / Boston, MA 02111-1307 / USA.
 *_______________________________________________________________________________
 */

package org.eidola.test;

import org.eidola.kernel.event.*;
import org.eidola.kernel.event.Event;
import org.eidola.kernel.event.EventQueue;
import org.eidola.kernel.event.EventListener;
import org.eidola.kernel.Container;
import org.eidola.kernel.NamedElement;
import org.eidola.util.*;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;
import javax.swing.*;

/**
	A toyishly simple test notation.
	
	@author Paul Cantrell
	@version [Development version]
*/

public class BoxTreeNotation
	extends Component
	implements EventListener
	{
	public BoxTreeNotation(Container top)
		throws Exception
		{
		this.top = top;
		q = new EventQueue("BoxTreeNotation queue");
		top.addListener(this, q);
		q.start();
		}
	
	public void handleEvent(Event e)
		{
//		System.out.println("+++ Got event " + e);
		if(e instanceof CompileCompleted)
			{
//			System.out.println("+++ Repaint");
			ContainerEvent sourceEvent = ((ContainerEvent) e).getPropagatedFrom();
			if(sourceEvent instanceof StructureChanged)
				// Race condition in this loop
				for(java.util.Iterator i = getDeepContents(sourceEvent.getSource()).iterator(); i.hasNext(); )
					{
					Container c = (Container) i.next();
					c.addListener(this, q);
//					System.out.println("+++ Listening to " + c);
					}
			repaint();
			}
		}
	
	static public Set getDeepContents(Container c)
		{
		return
			GraphUtil.reachableNodes(
				Collections.singleton(c),
				new GraphWalker()
					{
					public Collection getEdgesFrom(Object node)
						{ return ((Container) node).getCompilation().getContents(); }
					} );
		}
	
	public void paint(Graphics g)
		{
		Graphics2D g2 = (Graphics2D) g;
		g2.clearRect(0, 0, 10000, 10000);
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		paintLevel(g2, 3, 3, top, new LinkedList());
		}
	
	public double paintLevel(Graphics2D g2, double left, double top, Container c, LinkedList visited)
		{
		boolean cycle = visited.contains(c);
		visited.addLast(c);
		String name = (false && c instanceof NamedElement) ? ((NamedElement) c).getName() : c.toString();
		if(cycle)
			name = "... " + name + " ...";
		Rectangle2D bounds = g2.getFont().getStringBounds(name, g2.getFontRenderContext());
		double border = 3, hspace = 6, vspace = 12;
		double nameHeight = bounds.getHeight(), nameWidth = bounds.getWidth();
		double width = 0, firstx = Double.MAX_VALUE, lastx = Double.MIN_VALUE;
		double childTop = top + nameHeight + 2*border + vspace;
		boolean hasChildren = false;
		for(java.util.Iterator i = c.getCompilation().getContents().iterator(); i.hasNext(); )
			{
			hasChildren = true;
			Container cur = (Container) i.next();
			double curWidth = cycle ? 8 : paintLevel(g2, left + width, childTop, cur, visited);
			double linex = left + width + curWidth / 2;
			firstx = Math.min(firstx, linex);
			lastx  = Math.max(lastx,  linex);
			g2.setPaint(connectorPaint);
			g2.draw(new Line2D.Double(
				linex, childTop - vspace / 2,
				linex, childTop));
			width += curWidth;
			if(i.hasNext())
				width += hspace;
			}
		width = (width < nameWidth + 2 * border) ? nameWidth + 2 * border : width;
		if(hasChildren)
			{
			double underthingx = left + width / 2;
			firstx = Math.min(firstx, underthingx);
			lastx  = Math.max(lastx,  underthingx);
			g2.setPaint(connectorPaint);
			g2.draw(new Line2D.Double(
				firstx,      childTop - vspace / 2,
				lastx,       childTop - vspace / 2));
			g2.draw(new Line2D.Double(
				underthingx, childTop - vspace,
				underthingx, childTop - vspace / 2));
			}
		if(!cycle)
			{
			Shape box = new Rectangle2D.Double(
				left + (width - nameWidth) / 2 - border, top,
				nameWidth + border * 2, nameHeight + border * 2);
			g2.setPaint(boxFillPaint);
			g2.fill(box);
			g2.setPaint(boxPaint);
			g2.draw(box);
			}
		g2.setPaint(c.getCompilation().hasErrors() ? errorsPaint : noerrorsPaint);
		g2.drawString(
			name,
			(float) (left + (width - nameWidth) / 2),
			(float) (top - bounds.getY() + border));
		visited.removeLast();
		return width;
		}
	
	private Paint
		boxPaint = Color.darkGray,
		boxFillPaint = new Color(.89f, .92f, 1.0f),
		connectorPaint = new Color(.1f, 0f, .4f),
		noerrorsPaint = Color.black,
		errorsPaint = Color.red;
	private EventQueue q;
	private Container top;
	}


