/*______________________________________________________________________________
 * 
 * org.eidola.test.CommandLineInterface
 * 
 * Part of the Eidola Kernel Reference Implementation
 * See http://eidola.org for oodles of relevant fun!
 * 
 *______________________________________________________________________________
 * 
 * Copyright 2000-2001 Paul Cantrell
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License version 2, as published by the
 * Free Software Foundation. See the file LICENSE.html for more information.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY, including the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc. / 59 Temple
 * Place, Suite 330 / Boston, MA 02111-1307 / USA.
 *_______________________________________________________________________________
 */

package org.eidola.test;

import org.eidola.kernel.*;
import org.eidola.kernel.Compiler;
import org.eidola.kernel.event.*;
import org.eidola.kernel.event.EventListener;
import org.eidola.util.*;

import java.io.*;
import java.lang.Class;
import java.lang.reflect.*;
import java.util.*;

/**
	A very simplistic command line interface for testing purposes.
	
	@author Paul Cantrell
	@version [Development version]
*/

public class CommandLineInterface
	extends Thread
	implements EventListener
	{
	static public void main(String[] args)
		throws Exception
		{
		(new CommandLineInterface()).run();
		}
	
	public CommandLineInterface()
		{
		in = System.in;
		out = System.out;
		clear();
		}
	
	public void run()
		{
		BufferedReader in = new BufferedReader(new InputStreamReader(this.in));
		out.println("Eidola seat o' the pants command line interface!");
		out.println("?            for help");
		out.println("? example    for some sample commands");
		out.println("");
		while(true)
			try
				{
				String inputLine = in.readLine();
				if(inputLine == null)
					return;
				if(inputLine.indexOf('#') != -1)
					inputLine = inputLine.substring(0, inputLine.indexOf('#'));
				List cmdLine = parseCommandLine(inputLine);
				if(cmdLine.isEmpty())
					continue;
				history.add(inputLine.trim());
				
				Object first = cmdLine.get(0);
				if(first.equals("?"))
					help(cmdLine);
				else if(first.equals("new"))
					create(cmdLine);
				else if(first.equals("history"))
					history(cmdLine);
				else if(first.equals("list"))
					list(cmdLine);
				else if(first.equals("dump"))
					dump(cmdLine);
				else if(first.equals("debug"))
					debug(cmdLine);
				else if(first.equals("wait"))
					wait(cmdLine);
				else if(first.equals("clear"))
					clear(cmdLine);
				else if(first.equals("notate"))
					notate(cmdLine);
				else if(first.equals("quit") || first.equals("exit"))
					break;
				else
					{
					Object result = invoke(cmdLine);
					if(result != null)
						{
						if(result instanceof Collection)
							out.println("  [" + Util.collectionToString((Collection) result, " ") + "]");
						else
							out.println("  " + result);
						}
					}
				}
			catch(Throwable e)
				{
				e.printStackTrace(System.err);
				}
		}
	
	public List parseCommandLine(String commandLine)
		throws Throwable
		{
		List cmdList = new ArrayList();
		int curCmdStart = 0;
		commandLine += ' ';
		for(int n = 0; n < commandLine.length(); n++)
			{
			char c = commandLine.charAt(n);
			if(c == ' ' || c == '(' || c == '[')
				{
				if(curCmdStart < n)
					{
					String itemStr = commandLine.substring(curCmdStart, n);
					Object item = itemStr.equals("null") ? null : itemStr;
					try
						{
						item = containers.get(Integer.parseInt(itemStr));
						}
					catch(NumberFormatException nfe)
						{
						for(Iterator i = containers.iterator(); i.hasNext(); )
							{
							Container curContr = (Container) i.next();
							if(curContr.toString().equals(itemStr))
								{
								item = curContr;
								break;
								}
							}
						}
					cmdList.add(item);
					}
				curCmdStart = n + 1;
				}
			if(c == '[' || c == '(')
				{
				int m, balance = 1;
				for(m = n+1; balance > 0; m++)
					{
					char d = commandLine.charAt(m);
					if(d == '(' || d == '[')
						balance++;
					if(d == ')' || d == ']')
						balance--;
					}
				String listS = commandLine.substring(n+1, m-1);
				cmdList.add(c == '[' ? parseCommandLine(listS) : invoke(parseCommandLine(listS)));
				n = (curCmdStart = m) - 1;
				}
			}
		return cmdList;
		}
	
	public void help(List commandLine)
		{
		if(commandLine.size() <= 1)
			{
			out.println("   Commands:");
			out.println("      ?                          help (You're looking at it!)");
			out.println("      ? example                  show an example script");
			out.println("      ? <object>                 show info for container");
			out.println("      <object> <method> <params> method invocation (Sounds creepy, no?)");
			out.println("      list                       list all containers and errors");
			out.println("      new <contr type>           create a new container");
			out.println("      new <contr type> <name>    create a new named element");
			out.println("      dump <object>              dump container to screen");
			out.println("      dump                       dump all top-level containers");
			out.println("      debug [e|q|c|v]            toggle debug output for (e)vents /");
			out.println("                                   event (q)ueues / (c)ompiles / (v)iolations");
			out.println("      history                    show command history since last \"clear\"");
			out.println("      wait                       wait for a keypress");
			out.println("      clear                      clear the interface's container list");
			out.println("      notate <contr> <notation>  notate container using class from org.eidola.test");
			out.println("      quit|exit                  depart this convoluted mess of an interface");
			out.println("      # comment                  write secret messages to yourself and others");
			out.println("   Parameters:");
			out.println("      <object>     container number");
			out.println("                    | full container name");
			out.println("                    | \"null\"");
			out.println("                    | [<params>]                   <--a java.util.List");
			out.println("                    | (<object> <method> <params>) <--obj returned by method");
			out.println("      <name>       unquoted string literal");
			out.println("      <contr type> \"Package\" | \"Class\" | \"Function\" | \"Variable\"");
			out.println("      <method>     use \"? object\" to see options");
			out.println("      <params>     space-separated list of objects");
			return;
			}
		
		if(commandLine.get(1).toString().equals("example"))
			{
			out.println("   Example script (try it one line at a time):");
			out.println("      clear                  # clean slate");
			out.println("      new Class c            # create a class named 'c'");
			out.println("      ? c                    # show its methods");
			out.println("      new Variable v         # create a variable named 'v'");
			out.println("      v setType c            # set v's type to c,");
			out.println("      v setOwner c           #   and make it a public");
			out.println("      c addPublic c.v        #   member of c");
			out.println("      ? (c getCompilation)   # (...) evaluates to an object");
			out.println("      (c getCompilation) getContents  # show everything inside c");
			out.println("      new Class d            # create some more classes");
			out.println("      new Class e            #   so we can play with inheritance");
			out.println("      new Class f");
			out.println("      f setParents [d e]     # [d e] is a list");
			out.println("      debug e                # look under the hood for a moment");
			out.println("      debug c");
			out.println("      d addParent c");
			out.println("      debug e                # enough of that verbose output!");
			out.println("      debug c");
			out.println("      d addPublic c.v        # d inherits a member from c");
			out.println("      (f getCompilation) getGeneralizations");
			out.println("      new Variable v         # f overrides c.v");
			out.println("      f addPublic v");
			out.println("      v setOwner f");
			out.println("      f.v setType f");
			out.println("      dump                   # show structure we've built");
			out.println("      list                   # show errors (things need owners!)");
			out.println("      new Namespace          # create a namespace");
			out.println("      new Package p          # and a package");
			out.println("      notate p BoxTreeNotation  # show some graphics");
			out.println("      notate 6 JTreeNotation    #    while we do this");
			out.println("      p setOwner 6          # make p namespace's root");
			out.println("      6 setRoot p           # ('6' is the namespace)");
			out.println("      p addPublic c         # put everything inside p");
			out.println("      c setOwner p");
			out.println("      d setOwner p");
			out.println("      p addPublic p.d");
			out.println("      p addPrivate e");
			out.println("      p addPrivate f");
			out.println("      e setOwner p");
			out.println("      f setOwner p");
			return;
			}
		
		Object obj = commandLine.get(1);
		if(Container.class.isAssignableFrom(obj.getClass()))
			out.println("  Container #" + containers.indexOf(obj) + " " + obj + " (" + obj.getClass() + ")");
		else
			out.println("   " + obj.getClass());
		out.println("     Methods (excluding java.lang.Object methods) (* means not available from this interface):");
		Method[] methods = obj.getClass().getMethods();
		Arrays.sort(
			methods,
			new Comparator()
				{
				public int compare(Object o1, Object o2)
					{ return ((Method) o1).getName().compareTo(((Method) o2).getName()); }
				public boolean equals(Object other)
					{ return other == this; }
				}
			);
		for(int m = 0; m < methods.length; m++)
			{
			if(methods[m].getDeclaringClass() == Object.class)
				continue;
			java.lang.Class[] types = methods[m].getParameterTypes();
			boolean ok = true;
			for(int t = 0; t < types.length; t++)
				{
				if(!types[t].getName().equals("void")
				&& !Container.class.isAssignableFrom(types[t])
				&& !String.class.isAssignableFrom(types[t])
				&& !Collection.class.isAssignableFrom(types[t]))
					ok = false;
				}
			out.print("       " + (ok ? ' ' : '*') + methods[m].getName() + "(");
			Class[] params = methods[m].getParameterTypes();
			for(int p = 0; p < params.length; p++)
				{
				if(p > 0)
					out.print(", ");
				out.print(params[p].getName());
				}
			out.println(")    [" + methods[m].getDeclaringClass().getName() + "]");
			}
		}
	
	public void history(List commandLine)
		{
		for(Iterator i = history.iterator(); i.hasNext(); )
			out.println("  " + i.next());
		}
	
	public void list(List commandLine)
		{
		if(containers.size() == 0)
			out.println("  No containers -- use \"new {Contr Type}\" to create some!");
		for(int n = 0; n < containers.size(); n++)
			{
			out.println("  " + (n < 10 ? " " : "") + n + " " + containers.get(n));
			for(Iterator i = ((Container) containers.get(n)).getCompilation().getErrors().iterator(); i.hasNext(); )
				{
				Object v = i.next();
				out.println("     " + v);
				}
			}
		}
	
	public void create(List commandLine)
		throws Exception
		{
		Class type = Class.forName("org.eidola.kernel." + commandLine.get(1));
		Container newContr = (Container) type.newInstance();
		containers.add(newContr);
		newContr.addListener(this, eventq);
		out.println("  New " + commandLine.get(1) + " is container " + (containers.size()-1));
		if(commandLine.size() > 2)
			synchronized(newContr)
				{ ((NamedElement) newContr).setName(commandLine.get(2).toString()); }
		}
	
	public void dump(List commandLine)
		{
		if(commandLine.size() > 1)
			((Container) commandLine.get(1)).dump(3);
		else
			for(Iterator i = containers.iterator(); i.hasNext(); )
				{
				Container contr = (Container) i.next();
				if(contr instanceof Namespace
				 || ((Element) contr).getOwner() == null)
					contr.dump(3);
				}
		}
	
	public void debug(List commandLine)
		{
		String what = commandLine.get(1).toString();
		String whatname = null;
		boolean newDebug = false;
		if(what.equals("e"))
			{
			newDebug = EventBroadcaster.debugEvents = !EventBroadcaster.debugEvents;
			whatname = "events";
			}
		else if(what.equals("q"))
			{
			newDebug = EventQueue.debug = !EventQueue.debug;
			whatname = "event queues";
			}
		else if(what.equals("c"))
			{
			newDebug = Container.debugCompile = !Container.debugCompile;
			whatname = "compiles";
			}
		else if(what.equals("v"))
			{
			newDebug = debugViolations = !debugViolations;
			whatname = "violations";
			}
		if(whatname != null)
			out.println("  Debugging " + (newDebug ? "enabled" : "disabled")
				+ " for " + whatname);
		else
			out.println("  What's \"" + what + "\"?");
		}
	
	public void wait(List commandLine)
		throws Exception
		{
		out.print(" Press return to continue ");
		new BufferedReader(new InputStreamReader(in)).readLine();
		}
	
	public void clear()
		{
		errorVersions = new HashMap();
		containers = new ArrayList();
		history = new ArrayList();
		eventq = new EventQueue("cliQ");
		eventq.start();
		}
		
	public void clear(List commandLine)
		{ clear(); }
	
	public void notate(List commandLine)
		throws Exception
		{
		Container cntr = (Container) commandLine.get(1);
		Class type = Class.forName("org.eidola.test." + commandLine.get(2));
		java.awt.Component notation = (java.awt.Component)
			type.getConstructor(new Class[]{Container.class})
			    .newInstance(new Object[]{cntr});
		javax.swing.JFrame test = new javax.swing.JFrame(cntr + " (" + commandLine.get(2) + ")");
		test.getContentPane().setLayout(new java.awt.BorderLayout());
		test.getContentPane().add(notation, java.awt.BorderLayout.CENTER);
		test.setSize(new java.awt.Dimension(400,400));
		notationPosition += 20;
		test.setLocation(notationPosition, notationPosition);
        test.show();
		}
	
	public Object invoke(List commandLine)
		throws Throwable
		{
		Object target = commandLine.get(0);
		Object[] args = commandLine.subList(2, commandLine.size()).toArray();
		try
			{
			String methodName = commandLine.get(1).toString();
			Method[] methods = target.getClass().getMethods();
			for(int m = 0; m < methods.length; m++)
				if(methods[m].getName().equals(methodName))
					synchronized(target)
						{
						return methods[m].invoke(target, args);
						}
			throw new Exception("   " + target + " has no method named " + methodName);
			}
		catch(InvocationTargetException ite)
			{ throw ite.getTargetException(); }
		}
	
	public void handleEvent(Event event)
		{
		if(debugViolations)
			{
			Container source = ((ContainerEvent) event).getSource();
			if(event instanceof CompileCompleted
			  && checkErrorVersionShown(source))
				{
				List errors = source.getCompilation().getErrors();
				out.println("  #(v) " + source
					+ (errors.isEmpty()
						? " is violation-free!"
						: " has violations:"));
				for(Iterator i = errors.iterator(); i.hasNext(); )
					out.println("  #(v)   " + i.next());
				}
			}
		}
	
	public String toString()
		{
		return "CommandLineInterface";
		}
	
	private boolean checkErrorVersionShown(Container c)
		{
		synchronized(errorVersions)
			{
			long shown, current = c.getCompileVersion();
			try { shown = ((Long) errorVersions.get(c)).longValue(); }
			catch(NullPointerException npe) { shown = -1; }
			if(shown < current)
				{
				errorVersions.put(c, new Long(current));
				return true;
				}
			return false;
			}
		}
	
	public Container getContainer(int n)
		{
		return (Container) containers.get(n);
		}
	
	private Map errorVersions;
	private ArrayList containers;
	private List history;
	private EventQueue eventq;
	private boolean debugViolations = true;
	private PrintStream out;
	private InputStream in;
	private int notationPosition = 4;
	}





