/*______________________________________________________________________________
 * 
 * org.eidola.util.CollectionDiff
 * 
 * Part of the Eidola Kernel Reference Implementation
 * See http://eidola.org for oodles of relevant fun!
 * 
 *______________________________________________________________________________
 * 
 * Copyright 2000-2001 Paul Cantrell
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License version 2, as published by the
 * Free Software Foundation. See the file LICENSE.html for more information.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY, including the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc. / 59 Temple
 * Place, Suite 330 / Boston, MA 02111-1307 / USA.
 *_______________________________________________________________________________
 */

package org.eidola.util;

import java.util.*;

/**
	Finds the difference between two Collections.
	This class returns the delta using {@link java.util.Set},
	and therefore ignores duplicates and reordering.
	<p>
	CollectionDiff optionally creates internal copies of the input Collections.
	If you do not use this option, the results of CollectionDiff methods
	after a subsequent modification of the input collections are undefined.
	
	@author Paul Cantrell
	@version [Development version]
*/

public class CollectionDiff
	{
	/** Prepares a comparison of two collections without creating an
	 *  internal copy. */
	
	public CollectionDiff(Collection oldStuff, Collection newStuff)
		{ this(oldStuff, newStuff, false); }
	
	/** Prepares a comparison of two collections, optionally creating an
	 *  internal copy of the collections to allow subsequent modification. */
	
	public CollectionDiff(Collection oldStuff, Collection newStuff, boolean copy)
		{
		this.oldStuff = copy ? oldStuff : new HashSet(oldStuff);
		this.newStuff = copy ? newStuff : new HashSet(newStuff);
		}
	
	/** Returns the objects which are in newStuff but not oldStuff. */
	
	public Set getAdded()
		{
		if(added == null)
			{
			added = new HashSet(newStuff);
			added.removeAll(oldStuff);
			}
		return added;
		}
	
	/** Returns the objects which are in oldStuff but not newStuff. */
	
	public Set getRemoved()
		{
		if(removed == null)	
			{
			removed = new HashSet(oldStuff);
			removed.removeAll(newStuff);
			}
		return removed;
		}
	
	/** Returns the objects which are in both oldStuff and newStuff.
	 *  Objects which have changed position in a list are considered "same". */
	
	public Set getSame()
		{
		if(same == null)
			{
			same = new HashSet(oldStuff);
			same.retainAll(newStuff);
			}
		return same;
		}
	
	/** Returns true if the other object is a collection diff comparing equal collections. */
	
	public boolean equals(Object other)
		{
		if(other == null)
			return false;
		CollectionDiff otherdiff = (CollectionDiff) other;
		return otherdiff.oldStuff.equals(oldStuff)
			&& otherdiff.newStuff.equals(newStuff);
		}
	
	public int hashCode()
		{
		return oldStuff.hashCode() * 41 + newStuff.hashCode();
		}
	
	private Collection oldStuff, newStuff;
	private Set added, removed, same;
	}
