/*______________________________________________________________________________
 * 
 * org.eidola.util.EnumeratedType
 * 
 * Part of the Eidola Kernel Reference Implementation
 * See http://eidola.org for oodles of relevant fun!
 * 
 *______________________________________________________________________________
 * 
 * Copyright 2001 Kendall Helmstetter Gelner, Paul Cantrell
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License version 2, as published by the
 * Free Software Foundation. See the file LICENSE.html for more information.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY, including the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc. / 59 Temple
 * Place, Suite 330 / Boston, MA 02111-1307 / USA.
 *_______________________________________________________________________________
 */

package org.eidola.util;

import java.util.*;

/**
	An alternative to using hand-assigned integer constants for enumerated types.
	Subclass to create your own enumerated type:
	<p>
	<pre>public final class ColorType extends EnumeratedType
	{
		static public RED = new ColorType( "red" );
		static public GREEN = new ColorType( "green" );
		static public BLUE = new ColorType( "blue" );
		private ColorType( String type ) { super( type ); }
	}</pre>
	
	@author Kendall Helmstetter Gelner, Paul Cantrell
	@version [Development version]
*/

public abstract class EnumeratedType
{

	/** Creates a new enumerated type value. */
	protected EnumeratedType( String name )
		{ this.name = name; }

	/** Returns the name of this enumerated type value. */
	public String getName()
		{ return name; }

	private String name;

}
